//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                   Copyright 2005-2014, MetaQuotes Software Corp. |
//|                                              http://www.mql4.com |
//+------------------------------------------------------------------+
#property copyright   "2005-2014, MetaQuotes Software Corp."
#property link        "http://www.mql4.com"
#property description "Moving Average sample expert advisor"

enum ma_methods
{
defoult,EMA,Wilder,LWMA,SineWMA,TriMA,LSMA,SMMA,MMA,HMA,ZeroLagEMA,NonLagEMA,
};

enum bands_type
{
STD=1,ENVD,ATR,
};

extern string MA_set = "//////////////MA settings//////////////";
extern int AAB_MAPeriod = 32;
extern int AAB_MAShift = 0;
extern ma_methods AAB_MAMethod = Wilder;
extern ENUM_APPLIED_PRICE AAB_Price = PRICE_CLOSE;
extern double AAB_NLMADeviation = 0.0;
extern double AAB_NLMAPctFilter = 0.0;

extern string ba_set = "////////////BANDS settings/////////////";
extern bands_type AAB_BandsType = ENVD;
extern int AAB_STDPeriod = 20;
extern bool AAB_STDUseMAPeriod = TRUE;
extern double AAB_STDDevations = 1.5;
extern double AAB_ENVDevations = 0.3;
extern int AAB_ATRPeriod = 20;
extern bool AAB_ATRUseMAPeriod = TRUE;
extern double AAB_ATRDevations = 2.0;

extern string al_set = "/////////////Alerts settings/////////////";
extern bool EmailON = false;
extern bool SoundON = false;
extern bool AlertON = false;
extern int AlertRepeat = 3;
extern bool EndLessRepeat = FALSE;
extern string SoundFile = "Alert";
extern int Nbars = -1;

#define MAGICMA  20131111
//--- Inputs
input double Lots          =0.1;
input double MaximumRisk   =0.02;
input double DecreaseFactor=3;
input int    MovingPeriod  =12;
input int    MovingShift   =6;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//---
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//--- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//--- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//--- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
           {
            Print("Error in history!");
            break;
           }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL)
            continue;
         //---
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1)
         lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//--- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
 
   int    res;
//--- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//--- get Moving Average 
double buy_macurrent=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,0,1);
double buy_maprevious=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,0,1+1);
double sell_macurrent=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,1,1);
double sell_maprevious=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,1,1+1);
//--- sell conditions
if(sell_maprevious!=EMPTY_VALUE && sell_macurrent==EMPTY_VALUE && buy_macurrent!=EMPTY_VALUE ) 
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//--- buy conditions
   if(buy_maprevious!=EMPTY_VALUE && buy_macurrent==EMPTY_VALUE && sell_macurrent!=EMPTY_VALUE ) 
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//---
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//--- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//--- get Moving Average 

double buy_macurrent=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,0,1);
double buy_maprevious=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,0,1+1);
double sell_macurrent=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,1,1);
double sell_maprevious=iCustom(Symbol(),0,"qPallada_MainSignal_AllAverages",MA_set,AAB_MAPeriod,AAB_MAShift,AAB_MAMethod,AAB_Price,AAB_NLMADeviation,AAB_NLMAPctFilter,ba_set,AAB_BandsType,AAB_STDPeriod,AAB_STDUseMAPeriod,AAB_STDDevations,AAB_ENVDevations,AAB_ATRPeriod,AAB_ATRUseMAPeriod,AAB_ATRDevations,al_set,EmailON,SoundON,AlertON,AlertRepeat,EndLessRepeat,SoundFile,Nbars,1,1+1);
   
//---
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //--- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma && Close[1]<ma)
           {
            if(!OrderClose(OrderTicket(),OrderLots(),Bid,3,White))
               Print("OrderClose error ",GetLastError());
           }
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[1]<ma && Close[1]>ma)
           {
            if(!OrderClose(OrderTicket(),OrderLots(),Ask,3,White))
               Print("OrderClose error ",GetLastError());
           }
         break;
        }
     }
//---
  }
//+------------------------------------------------------------------+
//| OnTick function                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false)
      return;
//--- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//---
  }
//+------------------------------------------------------------------+
